

/**
 * @param {string} schemaName
 * @param {function} cb
 */
"use strict";

exports.loadSchema = loadSchema;
exports.validate = validate;

/**
 * Validates an object.
 * @param {object} obj The object to validate.
 * @param {schemaName} schema The name of the schema that is will be used
 * @param {function} cb
 * to validate [value].
 */
exports.validateAsync = validateAsync;
Object.defineProperty(exports, "__esModule", {
    value: true
});
"use strict";

var _helpers = require("./helpers");

var isBaseType = _helpers.isBaseType;
var isNullOrUndefined = _helpers.isNullOrUndefined;
var navigateSafely = _helpers.navigateSafely;

var validateContent = require("./content-validations").validateContent;

var _typeValidations = require("./type-validations");

var validateType = _typeValidations.validateType;
var extractType = _typeValidations.extractType;

/**
 * Schema cache.
 */
var schemas = {};

exports.schemas = schemas;
function validateProps(object, schema) {
    var props = {};
    var valid = true;

    for (var prop in schema) {
        if (schema.hasOwnProperty(prop)) {
            var _schema$prop = schema[prop];
            var type = _schema$prop.type;
            var validations = _schema$prop.validations;

            var result = validateProp(object[prop], type, validations);

            valid = valid && result.valid;

            props[prop] = result;
        }
    }
    return { valid: valid, props: props };
}

function ValidationResult() {
    this.typeErrors = [];
    this.validationErrors = [];
    this.valid = true;
}

function validateProp(value, itemType, validations) {

    var result = new ValidationResult();

    var type = extractType(itemType);

    // Validate the type of the prop if the prop type is a base type.
    // TODO Try to get this working with non-base types (prototypes?)
    if (itemType.indexOf("array") !== -1 && !Array.isArray(value) && (isBaseType(type) && !validateType(value, type))) {
        result.typeErrors.push("" + value + " is not of type " + type + ".");
    }

    result.validationErrors = validateContent(value, validations);

    if (Array.isArray(value)) {
        var isBase = isBaseType(type);
        result.itemErrors = [];

        for (var i = 0; i < value.length; i++) {
            var item = value[i];

            if (isBase) {
                if (!validateType(item, type)) {
                    result.typeErrors.push("" + item + " is not of type " + type + ".");
                }

                result.itemErrors.push(validateContent(value, validations));
            } else {
                var contentValidation = validateProps(item, schemas[type]);
                result.valid = result.valid && contentValidation.valid;
                result.itemErrors.push(contentValidation.props);
            }
        }
    } else if (!isBaseType(type) && !isNullOrUndefined(value)) {
        var contentValidation = validateProps(value, schemas[type]);
        result.valid = result.valid && contentValidation.valid;
        result.propertyErrors = contentValidation.props;
    }

    result.valid = result.valid && result.typeErrors.length === 0 && result.validationErrors.length === 0;

    return result;
}

/**
 * Parses a schema and returns any additional
 * schemas that may need to be loaded.
 * @param {object} schema
 * @return {array}
 */
function findAdditionalSchemas(schema) {
    var additionalSchemasToLoad = [];

    for (var prop in schema) {
        if (schema.hasOwnProperty(prop)) {
            var typeName = schema[prop].type;

            if (!isBaseType(extractType(typeName))) {
                additionalSchemasToLoad.push(extractType(schema[prop].type));
            } else if (!isBaseType(typeName)) {
                additionalSchemasToLoad.push(typeName);
            }
        }
    }

    return additionalSchemasToLoad;
}
function loadSchema(schemaName, done) {
    var additionalSchemas = [schemaName];

    var loadMoreSchemas = function (cb) {

        var schemaToLoad = additionalSchemas.pop();

        if (schemas[schemaToLoad]) {
            cb(null, schemaToLoad, schemas[schemaToLoad]);
            return;
        }

        var xhr = new XMLHttpRequest();

        xhr.open("get", "http://localhost:2106/validation/" + schemaToLoad);
        xhr.setRequestHeader("Accept", "application/json");

        xhr.addEventListener("load", function (e) {
            if (e.target.status === 200) {
                cb(null, schemaToLoad, JSON.parse(e.target.responseText));
            } else {
                cb(new Error("Schema " + schemaToLoad + " not found."), null);
            }
        });

        xhr.addEventListener("error", cb);
        xhr.send();
    };

    var parseSchema = function (err, loadedSchema, schema) {
        if (err) {
            throw err;
        }

        // Skip the additional load steps if the schema was
        // already loaded, because we load schemas ourselves.
        if (!schemas[loadedSchema]) {
            schemas[loadedSchema] = schema;
            additionalSchemas = additionalSchemas.concat(findAdditionalSchemas(schema));
        }

        if (additionalSchemas.length === 0) {
            done(null, schemas[schemaName]);
        } else {
            loadMoreSchemas(parseSchema);
        }
    };

    loadMoreSchemas(parseSchema);
}

function validate(obj, schema) {
    var result = {
        valid: true,
        modelState: {}
    };

    var validation = validateProps(obj, schema);

    result.modelState = validation.props;
    result.valid = validation.valid;

    return result;
}

function validateAsync(obj, schemaName, cb) {
    loadSchema(schemaName, function (err, schema) {
        if (err) {
            cb(err);return;
        }
        try {
            cb(null, validate(obj, schema));
        } catch (e) {
            cb(e);
        }
    });
}